#!/usr/local/bin/perl

use strict;

=head1 NAME

piclist - Making List of Pictures in a Directory

=cut

my $dir = $main::ENV{PATH_TRANSLATED}
  or die "BAD PATH_TRANSLATED: $ENV{PATH_TRANSLATED}";

my %Opt;

if ($dir =~ s#/[^/]+$##) {
  for (split /[&;]/, $ENV{QUERY_STRING}) {
    my ($name, $val) = split /=/, $_, 2;
    $Opt{$name} = defined $val ? $val : 1;
  }
} else {
  die "BAD PATH_TRANSLATED: $ENV{PATH_TRANSLATED}";
}


sub escape ($) {
  my $s = shift;
  $s =~ s/&/&amp;/g;
  $s =~ s/</&lt;/g;
  $s =~ s/>/&gt;/g;
  $s =~ s/"/&quot;/g;
  $s =~ s/'/&#x27;/g;
  $s;
}

sub rfc3339date ($) {
  my @gt = gmtime shift;
  sprintf '%04d-%02d-%02dT%02d:%02d:%02dZ',
          $gt[5] + 1900, $gt[4] + 1, @gt[3, 2, 1, 0];
}

sub filesize ($) {
  my $size = 0 + shift;
  if ($size > 2048) {
    $size /= 1024;
    if ($size > 2048) {
      $size /= 1024;
      sprintf '%.1f ᥬƥå', $size;
    } else {
      sprintf '%.1f ƥå', $size;
    }
  } else {
    $size . ' ƥå';
  }
}

my $dirpath = escape $ENV{REQUEST_URI};
$dirpath =~ s/\#.*$//;
$dirpath =~ s/\?.*$//;
$dirpath =~ s/,[^,]*$//g;
unless (-d $dir) {
  $dir =~ s#/+[^/]+$##;
  $dirpath =~ s#/[^/]+$#/#;
  $dirpath ||= '/';
} else {
  $dirpath =~ s#/LIST$##;
  $dirpath =~ s#/?$#/#;
}

opendir DIR, $dir or die "$dir: $!";
  my @all_files = sort grep {not /^\./ and /^[A-Za-z0-9._-]+$/}
                  (readdir DIR)[0..1000];
close DIR;
my @files = grep {/\.(?:jpe?g|png|ico|gif|mng|xbm|JPE?G)(?:\.gz)?$/} @all_files;
my @dirs = grep {$_ ne 'CVS' and -d $dir.'/'.$_} @all_files;

sub has_file ($) {
  my $name = shift;
  my $namelen = 1 + length $name;
  for (@all_files) {
    if ($name.'.' eq substr $_, 0, $namelen) {
      return 1;
    }
  }
  return 0;
}

sub preview_uri ($) {
  my $original_file_name = shift;
  $original_file_name =~ s/\..*$//;
  my $file_name = $original_file_name;
  if ($file_name =~ /-small$/) {
    return $file_name;
  } else {
    $file_name =~ s/-large$//;
    if (has_file $file_name . '-small') {
      return $file_name . '-small';
    } elsif (has_file $file_name) {
      return $file_name;
    } else {
      return $original_file_name;
    }
  }
}

my $title = '';

print STDOUT "Content-Type: text/html; charset=euc-jp\n\n";

my $linkelement = '<link rel="stylesheet" href="/s/image-list" media="all" />';

$| = '';
print <<EOH;
<!DOCTYPE html SYSTEM>
<html lang="ja">
<head>
<base href="http://suika.fam.cx$dirpath" />
<title>@{[$Opt{cframe} ? '' : qq<$dirpath >]}${title}</title>
${linkelement}
</head>
EOH

my $LISTq = q<>;
$LISTq .= q<;detail> if $Opt{detail};
$LISTq = substr $LISTq, 1;
$LISTq = 'LIST' . ($LISTq ? '?' . $LISTq : '');

if ($Opt{cframe}) {
  my $LISTqt = ($LISTq eq 'LIST' ? $LISTq . '?' : $LISTq . ';') . 'target=view';
  print qq{<frameset cols="25%,*">
             <frame src="$LISTqt" name="list" />
             <frame src="./" name="view" />
             <noframes>};
}

my $viewtarget = '';
my $listtarget = '';
my $parenttarget = '';
if ($Opt{target} =~ /^([a-z]+)$/) {
  $viewtarget = qq{ target="$1"};
  $listtarget = q{ target="_self"};
  $LISTq .= $LISTq eq 'LIST' ? qq{?target=$1} : qq{;target=$1};
  $parenttarget = q{ target="_parent"};
}

if ($Opt{detail}) {
  print qq{<body@{[$Opt{target}?' class="has-target"':'']}>
           <h1>${title}</h1>
           <div class="pictures detail">};
  
  for my $file_name (@files) {
    my $efile = escape $file_name;
    my $preview_uri = escape preview_uri $file_name;
    my $uri = $efile;
    $uri =~ s/\..+//g;
    my $id = uc $uri;
    my @cls = split /\./, lc $file_name;
    shift @cls;
    print qq{<div class="image-with-desc" id="FILE--$id">};
      print qq{<a href="$uri"$viewtarget>};
      print qq{<img src="$preview_uri" alt="" class="@{[join ' ', @cls, 's']}" /></a>};
      print qq{<dl><dt>URI</dt><dd>};
      print qq{<code class="uri">&lt;<a href="$uri"$viewtarget>$uri</a>&gt;</code></dd>};
      print qq{<dt>ե̾</dt><dd>};
      print qq{<code class="file"><a href="$efile"$viewtarget>$efile</a></code></dd>};
      print qq{<dt></dt><dd>};
      print rfc3339date ([stat $dir.'/'.$file_name]->[9]);
      print qq{<dt>礭</dt><dd>};
      print filesize ([stat $dir.'/'.$file_name]->[7]);
      print qq{</dd>};
      print qq{</dl>};
    print q{</div>};
  }

  my @videos = grep {/\.(?:avi|mpe?g|mp3|wav|mid|swf)(?:\.gz)?$/i} @all_files;
  for my $file_name (@videos) {
    my $efile = escape $file_name;
    my $uri = $efile;
    $uri =~ s/\..+//g;
    print q{<div class="image-with-desc">};
      print qq{<a href="$uri"$viewtarget><img src="/~wakaba/archive/2005/movie-1" alt="" /></a>};
      print qq{<dl><dt>URI</dt><dd>};
      print qq{<code class="uri">&lt;<a href="$uri"$viewtarget>$uri</a>&gt;</code></dd>};
      print qq{<dt>ե̾</dt><dd>};
      print qq{<code class="file"><a href="$efile"$viewtarget>$efile</a></code></dd>};
      print qq{<dt></dt><dd>};
      print rfc3339date ([stat $dir.'/'.$file_name]->[9]);
      print qq{</dd>};
      print qq{<dt>礭</dt><dd>};
      print filesize ([stat $dir.'/'.$file_name]->[7]);
      print qq{</dd>};
      print qq{</dl>};
    print q{</div>};
  }

  for my $dir_name (@dirs) {
    my $edir = escape $dir_name;
    print q{<div class="dir dir-with-desc">};
      if (-f $dir . '/' . $dir_name . '/favicon.png' or
          -f $dir . '/' . $dir_name . '/favicon.ico') {
        print qq{<img src="$edir/favicon" alt="" class="mini-icon" />};
      } else {
        print qq{<img src="/icons/folder" alt="" class="mini-icon" />};
      }
      print qq{<code class="file"><a href="$edir/$LISTq"$listtarget>$edir/</a></code>};
    print q{</div>};
  }

  for (['cover', 'ɽ', 'start'],
       ['introduction', 'Ϥ', 'start'],
       ['intro', 'Ϥ', 'start'],
       ['README', 'Ϥ'],
       ['contents', 'ܼ', 'contents'],
       ['list', '', 'contents'],
       ['description', ''],
       ['index', '', 'index'],
       ['latest', 'ǿ'],
       ['current', '']) {
    if (has_file $_->[0]) {
      print q{<div class="file file-with-desc">};
        print q{<img src="/icons/layout" alt="" class="mini-icon" />};
        print qq{<a href="$_->[0]" rel="$_->[2]"$viewtarget>$_->[1]</a>};
      print q{</div>};
    }
  }

  print q{<div class="dir-up dir-with-desc">};
    print q{<img src="/icons/forward" alt="" class="mini-icon" />};
    print qq{<a href="../$LISTq" rel="up"$listtarget>γ</a>};
  print q{</div>};
  
  print q{</div>};

} else {  ## Normal Listing Mode
  print qq{<body@{[$Opt{target}?' class="has-target"':'']}>
           <h1>${title}</h1>
           <div class="pictures">};
  my $imgsattr = ' class="s"';

  for my $file_name (@files) {
    my $uri = escape $file_name;
    $uri =~ s/\..+$//g;
    my $preview_uri = escape preview_uri $file_name;
    print '<a href="'.$uri.'"'.$viewtarget.'>';
    print '<img src="'.$preview_uri.'" alt="'.$uri.'"'.$imgsattr.' />';
    print "</a>\n";
  }

  print q{</div>};

  print q{<ul>};
  for my $dir_name (@dirs) {
    my $edir = escape $dir_name;
    print q{<li class="dir">};
      print qq{<code class="file"><a href="$edir/$LISTq"$listtarget>$edir/</a></code>};
    print q{</li>};
  }
  print qq{<li class="dir-up"><a href="../$LISTq" rel="up"$listtarget>γ</a></li></ul>};
}

my $cvslink = '';
if (-d $dir . '/CVS') {
  if (-f $dir . '/CVS/Root') {
    open my $root, '<', $dir . '/CVS/Root';
    if (<$root> =~ m#^(/home/cvs|/home/wakaba/pub/cvs)/?$#) {
      my $rpath = $1;
      if (-f $dir . '/CVS/Repository') {
        open my $repo, '<', $dir . '/CVS/Repository';
        my $reppath = escape <$repo>;
        $reppath =~ tr/\x0A\x0D//d;
        if ($reppath) {
          $cvslink = qq{ <a href="/gate/cvs/$reppath/@{[
                         {q[/home/cvs] => '',
                          q[/home/wakaba/pub/cvs] => '?cvsroot=Wakaba'}->{$rpath}
                       ]}" rel="history"$parenttarget>γؤ</a>};
        }
      }
    }
  }
}

print <<EOH;

<div class="footer">
<div class="navigation">
[<a href="/" rel="home"$parenttarget>/</a>]
[<a href="." rel="contents"$parenttarget>γ</a>$cvslink]
[ (<a href="LIST" rel="alternate"$parenttarget>ʰ</a>, 
<a href="LIST?cframe" rel="alternate"$parenttarget>ʰסʬ</a>,
<a href="LIST?detail" rel="alternate"$parenttarget>ܺ</a>,
<a href="LIST?detail;cframe" rel="alternate"$parenttarget>ܺ١ʬ</a>)]
</div>
</div>
</body>
EOH

print q{</noframes></frameset>} if $Opt{cframe};

print q{</html>};

1;

__END__



=head1 CHANGES

2005-02-26  Wakaba <w@suika.fam.cx>

        - Frame mode implemented.

2005-02-25  Wakaba <w@suika.fam.cx>

        - Use external style sheet. 
        - Detail mode implemented.

2001-06-25  Wakaba <wakaba@61.201.226.127>

	- In default, images are sized by stylesheet.  When ?realsize=1,
	  images are not specified its size.
	- Images are linked to itself.

2001-05-17  Wakaba

	- New File.

=head1 LICENSE

Public Domain. 

=cut

# $Date: 2007/11/22 12:50:09 $
